; One Button Home - Master Unit .ASM  --  2313 micro-controller project

;January 19, 2007


; UNIT Number 10

; ***** / Gives out one second pulse and detects heavy current draw on main
; ***** /   Data/Power Line.  This, in turn, generates a 1/10,000 sec. pulse 
; ***** /   upon that line. (50 usec)

; ***** / Pulses for specific touch-plate units are 1/1,000 second apart.

; ***** / 3.58 MHz Xtl for clock


; ***** / NOTE:  All interruptible delays have been eliminated


.include "2313def.inc"

; R16 general purpose temporary use register
; R20 contains downcnt.
; R21 contains TPCounter count


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt
rjmp	START1
rjmp	START1

	
START1:			;set up stack pointer and initialize cold start

ldi 	r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16			; Initialize stack pointer to end of internal RAM

ser		r16		;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output

clr		r16
out		PortB,r16	;LEDs are off when port pins are low

ldi		R16,226		;Set bit 0, 2, 3, 4 as inputs ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input
out		PORTD,R16	;execute to set out ports high, and in port with
			; no input resistor

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
			;and bits 0,1,2 select prescaling source as 64 of clock

ldi		r16,218
out		OCR1AH,r16
ldi		r16,125		;default selected at 125 for this particular crystal
out		OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
			; -- clock adj. to run slow (for Corr Purp)
			
rcall	ZeroCommandMemory		;zero all Command Memory locations

clr		r30		;clear delay-on counter
clr		r31		;clear delay-on counter

clr		r18		;zero all flags in register r18
sts		$B3,r18		;zero atomic missed day counter
sts		$B2,r18		;zero atomic clock countdown


InitializeClkTime:		;starts clock at 8:00 a.m.

ldi		r16,1
sts		$63,r16			;put Sunday(1) in as start of day-of-week

ldi		R16,224			;8:00 A.M. low byte
sts		$61,r16		

ldi		r16,1			;8:00 A.M. high byte
sts		$62,R16

clr		r15			;clear seconds counter to zero
inc		r15

ldi		r16,13
sts		$64,r16			 ;default load of clock correction - add 13 sec.
	
ldi		r16,8
sts		$65,r16			; $65 =permanent store for every other day, 
				; default = 8 = (every 8th day) 
sts		$66,r16			; $66 =temporary store for $65 above, 
				; 0= no correction, 1 = every day
				; 2 = every other day, 3 = every 3rd day, etc.
				; default = add 0 second each day

ldi		r26,$006A		;initialize at start of memory locations
				
ldi		r16,$6C			;set up start of recording area for data
sts		$67,r16			;bookmark start of command memory

ldi		r16,20
sts		$6B,r16			;Unit number 10 (unit number times 2)

ldi		r16,50			;CDS Photocell unit number to control 25 (X2)
sts		$B4,r16

clr		r16
sts		$AE,r16
sts		$AF,r16			;zero delay-on counter
ldi		r16,7
mov		r8,r16			;put value of 7 into 7X counter
clr		r22			;zero 4:16 counter

bset	7			;enable global interrupts

rcall	Delay2310		;delay bootup so touch plate circuit can charge


;------------------------------------



MainLoop:

sbrc	r18,3			;skip if bit 3 (8) is clear (continue in main loop)
rjmp	ComputerLoop

sbic	PIND,0x00		;skip if pin 0, port D touch plate touched			; is NOT touched (set)	
rjmp	ML3


ML2:

rjmp	DecodeMLTouchPlate


ML2b:

cbr		r18,4			;clear bit 2(4), reset second flag



ML3:

sbrc	r18,2			;skip if bit 2 (4) is clear (one second flag)
rjmp	GiveCommands	

rjmp	MainLoop


;-------------------------------------------


ComputerLoop:

sbic	PIND,0		;skip if touch plate touched
rjmp	MainLoop

rcall	ButtonPushTwo

rjmp	MainLoop



;----------------------------


DecodeMLTouchPlate:

rcall	ButtonPush	;check for entry

cpi		r20,1		;checks to see if LONG Count has been recorded
brsh	TPLongCount	;branch if same or higher than 1

			
TPShortCount:		;decode short counts


TPSC1:

cpi		r21,1		;#1 short = Togle AC on/off
brne	TPSC2

rcall	ToggleAC
rjmp	TPCEnd


TPSC2:

cpi		r21,2		;#2 short = turn light on for one hour
brne	TPSC3


TPS1:

ldi		r16,16
sts		$AE,r16
ldi		r16,14
sts		$AF,r16		;1 hour on time calculation
rjmp	TPSOut


TPSC3:

cpi		r21,3		;#3 short = turn light on for three hours
brne	TPSCOther

ldi		r16,48
sts		$AE,r16
ldi		r16,42
sts		$AF,r16		;3 hour on time calculation
rjmp	TPSOut


TPSCOther:		;  All other selections toggle other units on or off


lsl		r21			;multiply by two
sts		$69,r21		;store command into $69, immediate command store

sbr		r18,16		;set bit 4(16), immediate flag to on

rjmp	TPCEnd



TPSout:

sbi		PortB,2		;turn on relay


TPCEnd:

sbi		PortB,0		;on red led
rcall	Delay165
cbi		PortB,0		;blink red led once

rjmp	ML2b


;----------------------------


TPLongCount:

cpi		r20,10		;Is long count >=10 ? (sets time/command mode flag)
brsh	TPLC1

cpi		r20,3		;Is long count >=3 ?  (Turn another switch on/off)
brsh	TPLC2

rjmp 	TPCEnd



TPLC1:			;set time/command flag active

sbr		r18,8		;set bit 3(8) to on
rjmp	TPCEnd


TPLC2:			;Turn another switch off

rcall	ButtonPush
cpi		r20,1
brsh	TPCEnd


TPLC2a:

cpi		r21,105		;limit number
brlo	TPLC2b
rjmp	TPCEnd


TPLC2b:

sbr		r18,16		;set bit 4(16), immediate flag to on

lsl		r21			;multiply by two

inc		r21			;it is off command so extra cycle added
sts		$69,r21		;store command into $69, immediate command store

rjmp	TPCEnd


;-----------------------------


GiveCommands:


cbr		r18,4		;clear one second flag, bit2(4)

rcall	UsecPulse	;issue 50 usecond START pulse on power/data line

sbrs	r18,4		;skip if bit 4(16), immediate command is set
rjmp	GC2

GC1:

lds		r19,$69		;get immediate command from SRAM
cbr		r18,16		;clear immediate flag, bit 4(16)
rjmp	GC4


GC2:

sbrs	r18,5		;skip if bit 5(32), timed command flag is set
rjmp	GCDN


GC3:

lds		r24,$6A		;get timed command from SRAM

cpi		r24,20		;find if it is a toggle command for Unit #10
breq	GC3a

cpi		r24,21		;find if it is an off command for unit #10
breq	GC3b
rjmp	GC3c


GC3a:			;toggle relay

rcall	ToggleAC	;Toggle AC relay on and off		
rjmp	GC3c


GC3b:

cbi		PortB,2		;off relay


GC3c:

cbr		r18,32		;clear timed command flag, bit 5 (32)


GC4:			;delay the pulse less than one-half cycle to positiion

ldi		r16,200		;test value of 200 entered (was 250)


GC4a:

sbis	PIND,0		;skip if touch plate not touched
rjmp	MainLoop	

dec		r16
brne	GC4a

ldi		r16,182

GC4b:

dec		r16	
brne	GC4b

;--------------------------------


GC4c:			;Issue number of delay cycles

clr		r16
ldi		r17,2		;established delay number

GC4d:

sbis	PIND,0		;skip if touch plate not touched
rjmp	MainLoop

nop
nop

dec		r16
brne	GC4d

dec		r17
brne	GC4d


GC4e:

dec		r19
brne	GC4c

rcall	UsecPulse	;issue 50 usec pulse for unit command


;------------------	;This secion looks for "ON," 2nd pulse
			;IF detected, the Sonalert beeps out unit number

ReceiveResultPulse:
			
clr	r16			;delay of 286 usec before searching for pulse

RRP1:

nop
dec		r16
brne	RRP1		;execute 286 usec delay before continuing
			
clr		r16


RRP2:

sbic	PIND,2		;skip if data line pulse clear -- not present
rjmp	BeepUnitNo
nop
nop
dec		r16
brne	RRP2		;500 usecond window to detect second pulse

rjmp	BUN6


BeepUnitNo:		;Beep unit number out on Sonalert
			;Part of PCB routine

lds		r19,$69		;get immediate command unit number
lsr		r19			;divide by two to get correct unit number


BUN1:

cpi		r19,10
brsh	BUN5


BUN2:

tst		r19
breq	BUN6

dec		r19
sbi		PortB,4		;on Sonalert
rcall	Delay165
cbi		PortB,4		;off Sonalert
rcall	Delay495

rjmp	BUN2


BUN5:		;decrement by 10

subi	r19,10

sbi		PortB,4		;On sonalert
rcall	Delay495
cbi		PortB,4
rcall	Delay495	; generate long beep for 10s count

rjmp	BUN1


BUN6:

rjmp	MainLoop


;------------------


GCDN:			;CDS photocell command section

tst		r9		;test to see if r9 not zero (flag set for CDS photocell)
breq	PCD		;continue processing

clr		r9		;reset CDS photocell flag to zero
lds		r19,$B5		;get unit number toggle or off command from SRAM 

rjmp	GC4		;execute TOGGLE or OFF command


PCD:			;PowerCommandLineDecode:

push	r28
push	r29

lds		r19,$6B		;get unit number (unit number time 2)
clr		r28			;zero Z register
clr		r29


PCD1:

clr		r16
ldi		r17,2


PCD2:

sbis	PIND,0		;skip if touch plate not touched
rjmp	PCDEnd

nop	
nop

dec		r16
brne	PCD2

dec		r17	
brne	PCD2

dec		r19		;number of unit number loops
brne	PCD1


PCD3:

adiw	r28,1		;increment on/off relay count (1)

sbic	PIND,2		;skip if bit 2(4), pwr/data line is clear (no pulse)(2T) 

rjmp	PCD4		;pulse is detected (2)

cpi		r29,4		;max number for count (1)
brsh	PCD4a		;jump out because of max count (1F)

rjmp	PCD3		;loop back(2)


PCD4a:

rcall	DelayPoint1	; .1 second delay -- pass through rest of detects
rjmp	PCDEnd


PCD4:			;decode on or off command

clr		r16
sts		$AE,r16
sts		$AF,r16		;clear count in delay on counter
cpi		r29,2
brsh	PCD4ac

rcall	ToggleAC	;toggle AC relay -- it is an even count


;------			;This section adds second pulse IF relay is ON


SendResultPulse:

sbis	PortB,2		;skip if relay, PortB,2 is set (on)
rjmp	PCD4a

clr		r16


SRP1:			;provides 430 usec delay before 2nd pulse is sent

nop	
nop
nop
dec		r16
brne	SRP1		;loop back until 430 usec delay is fulfilled

rcall	UsecPulse	;send out 2nd pulse indicating this circuit is ON
rjmp	PCD4a


PCD4ac:

cbi		PortB,2		;off relay


PCDEnd:

pop		r29
pop		r28

rjmp	MainLoop


;----------------------------


ButtonPush:	;Replaces old DetectCnt - Updated 2/9/2005

push	r23
push	r30


BP12:

clr		r23		;set up > 2 minute delay timeout
ldi		r16,8		;(seven full cycles)


BP12a:

sbis	PIND,0		;skip if touch plate not touched
rjmp	BP17		;recycle until button is pushed


rcall	Delay40th	;1/40th second delay routine

dec		r23		;delay of >2 minutes in event touch plate abandoned
brne	BP12a

dec		r16
brne	BP12a

cbr		r18,8		;clear bit 3(8) - flag (return to main loop)
rjmp	BPEnd


BP17:

clr		r20
clr		r21


BP15:

ldi		r23,80	; Two second wait delay = 80 (delay for Down Count)


BP1:

sbic	PIND,0	;skip if port D, bit 2, touch plate input, is low.
rjmp	BP11

rcall	Delay40th

dec		r23
brne	BP1


BP3:

sbi		PortB,0		;flash RED LED
rcall	Delay165
cbi		PortB,0

inc		r20		;step long count one count

ldi		r23,30		;reset to less than 1 sec down count
rjmp	BP1


BP11:

cpi		r23,62		;compare count in r23 for long touch
brlo	BP10		;branch if lower (long touch - dah)

inc		r21		;step short count one count - a dit short touch command	

rjmp	BP9


BP8:

ldi		r23,50		;reset to 1 1/4 second pause between touches count
rjmp	BP5


BP10:			;increment by 10 because it is dah, long touch

ldi		r16,10
add		r21,r16


BP9:

cpi		r21,100		;be sure count in r21 is below 100
brlo	BP8

ldi		r21,0		;zero r21 - count is invalid
rjmp	BP8


BP5:

rcall	Delay40th
dec		r23
brne	BP6


BP16:

tst		r20		;test for zero
breq    BPEnd

clr		r21		;zero r21 since r20 has holddown count data
rjmp	BPEnd		; (only one register should contain data)


BP6:

sbic	PIND,0	;skip if port D, bit 2, touch plate input, is low.
rjmp	BP5


BP7:

rcall	Delay40th
rjmp	BP15


BPEnd:

pop		r30
pop		r23

ret


;----------------------------


ButtonPushTwo:		;includes string of functions requiring attention

rcall	ButtonPush	
	
			;END of ButtonPush --- now comes the decoding

cpi		r20,1		;checks to see if LONG Count has been recorded
brsh	LongCount	;branch if same or higher than 1

; The following are short counts

ShortCount:


rjmp	LCCEnd


;-----------------------------



LongCount:		;series for long counts



;-----------------------

LCC2:

cpi		r20,2
breq	LCC2sp1
rjmp	LCC3


LCC2sp1:

rcall	DisplayClock

rjmp	LCCEnd

;---------------------------------


LCC3:

cpi		r20,3
breq	LCC3a
rjmp	LCC4			;Set new default number for Unit Number


LCC3a:

push	r13

clr		r13
rcall	TwoGreenFlash	;Signal that two entries are needed

rcall	ButtonPush
rcall	Mult10		;multiply r20 value by 10
add		r13,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonPush
add		r13,r20		;add in LSD of count -- data is now complete

lsl		r13			;multiply value by TWO
sts		$6B,r13		;store new unit number in $6B (1 to 50)

pop		r13

rjmp	LCCEnd

;----------------------------------


LCC4:			;Change unit number for CDS photocell auto on/off

cpi		r20,4
breq	LCC4a
rjmp	LCC5			;Set new default number for Unit Number


LCC4a:

push	r13

clr		r13
rcall	TwoGreenFlash	;Signal that two entries are needed

rcall	ButtonPush
rcall	Mult10		;multiply r20 value by 10
add		r13,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonPush
add		r13,r20		;add in LSD of count -- data is now complete

lsl		r13		;multiply value by TWO
sts		$B4,r13		;store new unit number in $B4 (1 to 50)

pop		r13

rjmp	LCCEnd

;---------------------------------------


LCC5:			

cpi		r20,5		;ZERO command memory
			; Manually enters time into command memory
breq	LCC5a
rjmp	LCC6


LCC5a:

rcall	EnterAlarmTimeData

rjmp	LCCEnd

;------------------------------------


LCC6:

cpi		r20,6		; Manuarlly add additional times into command memory
			;   Command memory NOT zeroed!
breq	LCC6a
rjmp	LCC7


LCC6a:

rcall	EnterAlarmTimeData2

rjmp	LCCEnd


;---------------------------


LCC7:				;Set clock

cpi		r20,7
breq	LCC7a
rjmp	LCC10


LCC7a:

rcall	SetClock
rjmp	LCCEnd


;---------------------------------


LCC10:			;Set LSB for compare register 3.58 countdown

cpi		r20,10
breq	LCC10a
rjmp	LCC11

LCC10a:

rcall	SetCompareRegister	;set 3.58 clock compare register during interrupts

rjmp	LCCEnd


;--------------------------------

LCC11:

cpi		r20,11		;Set number of seconds correction and how often
breq	LCC11a
rjmp	LCCEnd

LCC11a:

rcall	SecondsCorrection	;set correction
rjmp	LCCEnd

;------------------------------


LCCEnd:

cbr		r18,8		;set r18, bit 3(8) for return to main loop
rcall	Wag

ret

;---------------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16
push	r30
push	r31

in		r16,SREG	;save flags
sts		$60,r16

rcall	NightDetect	;check for daylight/night

rcall	AtomicClockReset	;check for >1 second reset pulse

sbrc	r18,3		;skip if bit 3(8) mainloop/time-command loop flag is clear
rcall	UsecPulse	;give pulse out during computer mode




sbr		r18,4		;set bits 2 (4) -- second flag 
			;(set each second)


lds		r30,$AE
lds		r31,$AF		;get delay-on count from SRAM

mov		r16,r30
or		r16,r31
breq	CC1b		;bypass decrement of Z register if zero


sbiw	r30,1		;decrement r30,31, Z register
sts		$AE,r30
sts		$AF,r31

or		r30,r31
breq	CC1a		;off relay if zero
rjmp	CC1b


CC1a:

cbi		PortB,2		;off relay


CC1b:

inc		r15			;step one count of SECONDS counter
mov		r16,r15

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr		r15		;zero r15 -- SECONDS counter

sbr		r18,64		;set bit 6(64) -- the NEW MINUTE flag



AtomicClockDuplicateStop:

lds		r16,$B2		;get value from SRAM for atomic clock 
tst		r16
breq	CCDI1Cntr

dec		r16
sts		$B2,r16		;decrement and store


CCDI1Cntr:

lds		r16,$61		;load low minutes byte into R16

inc		r16
sts		$61,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$62		;high byte minutes
inc		r16		;increment high byte minutes
sts		$62,r16


CCDI1a:

lds		r16,$62		;check high byte if it is 5
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$61
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

lds		r16,$B3		;get days missed count
inc		r16
sts		$B3,r16		;increment and store new value for days missed

clr		r16
sts		$61,r16
sts		$62,r16

lds		r16,$63
inc		r16
sts		$63,r16

cpi		r16,8

breq	CCDI1d


CCDI1e:

rcall	ClockCorrection	;call clock correction (midnight corr)

rjmp	CCDIalarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$63,r16
rjmp	CCDI1e		;to continue


CCDIalarmExit:

rcall	CheckAllAlarms	


CCDIExit:		;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r31
pop		r30	
pop		r16

reti


;-----------------------------


CheckAllAlarms:		;check entire list of alarm commands for match

push	r26
push	r30

ldi		r26,$6C		;low byte -- start of alarm memory - x register


CAA1:			;check for end of list

lds		r16,$67		;get present bookmark for command memory pointer

cp		r26,r16		;subract r16 from r26 to examine result
breq	CAAEnd		;jump out if equal zero - no further checks needed

ld		r30,x+		;load low command byte into r30 and post decrement
lds		r16,$61		;load LSD into r16 for comparison
cp		r16,r30
brne	CAAPass2

ld		r30,x		;get MSD plus day-of-week
andi	r30,7		;mask out all but MSD data
lds		r16,$62		;load MSD data into r16 for comparison
cp		r16,r30
brne	CAAPass2

ld		r30,x+		;get MSD plus day-of-week data for dow comparison
lsr		r30
lsr		r30
lsr		r30		;position day-of-week data for comparison

cpi		r30,0
breq	CAA3


CAA2:

lds		r16,$63		;check for day-of-week match
cp		r16,r30
brne	CAAPass1


CAA3:

ld		r30,x+		;get unit number -- toggle and off - post decrement

sts		$6A,r30		;store into Timed Command store area
sbr		r18,32		;set timed command flag r18, bit 5(32)

rjmp	CAAEnd		;exit - no other commands should be found


CAAPass2:

inc		r26


CAAPass1:

inc		r26

rjmp	CAA1		;go back for next check


CAAEnd:

pop		r30
pop		r26

ret

;-------------------------------------------------------


ClockCorrection:	;subtracts seconds in $66 to time at minight 
			; determined by $65 & $66
lds		r16,$66
tst		r16		;if zero, no correction is needed
			;1= every day, 2= every other day, etc.

breq	CCExit		;exit routine with no action

dec		R16
sts		$66,r16		;store new value in variable

breq	CC3		;it is time for update of second counter

ret


CC3:

lds		r16,$64		;seconds correction from permanent store
mov		r15,r16		;put in seconds store area to correct time

lds		r16,$65		;get permanently stored value for interval correction
sts		$66,r16		;store new value in variable


CCExit:

ret

;----------------------------------------------


VarClockFlash:		;flashes red or green LED (G=AM R=PM)
			;R16 contains number of flashes
push	r17

rcall	Delay2310	;delay before flashes start

lds		r17,$62		;check if AM or PM time (less than 720 = AM)
cpi		r17,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r17,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

lds		r17,$61		;compare low byte for less than 208
cpi		r17,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:			; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,1		;turn on GREEN LED (AM)
rcall	Delay330	;on time
cbi		PortB,1		;off Green LED

sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	Delay495		;off time
dec		r16
rjmp	VCF1


VCF1ab:			;gives double flash

rcall	Delay165	;small pause
sbi		PortB,1		;on green LED
rcall	Delay330	;on time
cbi		PortB,1		;off Green LED
rjmp	VCF1ac


VCF2:			; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,0		;turn on RED LED
rcall	Delay330	;on time delay
cbi		PortB,0		;turn off RED LED
sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	Delay495		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:			;give double red flash

rcall	Delay165	;short off delay
sbi		PortB,0		;on Red LED
rcall	Delay330	;on time delay for second flash
cbi		PortB,0		;off Red LED
rjmp	VCF2ac


VCF2a:

pop		r17
cbr		r18,2		;clear bit 1 (2) - double flash flag
ret

;-------------------------------------------


DisplayClock:		;displays time

rcall	Wag		;begin clock display with wag of LEDs

clr		r6		;zero hours count
clr		r7		;zero MSD of minutes count

bclr	7		;turn off interrupts

lds		r28,$61		;load low byte into r28
lds		r29,$62		;load high byte into r29

bset	7		;turn interrupts back on

tst		r29		;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12		;flash out 12 a.m.

rcall	VarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:			;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6		;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3		;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4		;restore low byte in r28
			;at this point, minutes remainder is in low byte
			; (r29 will be zero)
			

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:			;flash out time

rcall	VarClockFlash


DC7:			;decode minutes with MSD double flashed

mov		r16,r28
inc		r7		;use r7 for MSD count
subi	r28,10		;get remaining minutes
brcs	DC7a

rjmp	DC7		;loop back for another subtraction


DC7a:

dec		r7		;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r18,2		;set bit 1 (2) for double flash
rcall	VarClockFlash	;flash out MSD
cbr		r18,2		;clear bit 1 (2) of double flash

mov		r16,r28
rcall	VarClockFlash	;flash out LSDs

rcall	Delay2310	;delay after display

ret


;---------------------------


ZeroCommandMemory:	;zero all command storage locations

push 	r1
		
clr		r1
ldi		r26,$6C	;low start byte for start of storage locations
sts		$67,r26	;store new bookmark for start of entries

ldi		r16,66	;for zeroing 66 locations

ZCM:

st		x+,r1	;zero location

dec		r16	
brne	ZCM

pop		r1
ret

;----------------------------------


ToggleAC:		;toggles AC power on/off

sbis	PORTB,2		;skip if bit 2 (AC) is set
rjmp	TAC1

cbi		PortB,2
ret

TAC1:

sbi		PortB,2		;clear bit 2 (off power)
clr		r16
sts		$AE,r16
sts		$AF,r16		;zero count for delay-on count
cbr		r18,128		;clear cycle lights at night flag, bit 7(128)

ret


;--------------------------

SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1

clr		r1
rcall	TwoGreenFlash
rcall	OneGreenFlash	;total of three flashes indicating multiple inputs

rcall	ButtonPush
rcall	Mult100		;multiply r20 value by 100
mov		r1,r20		;set 100's column

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonPush
rcall	Mult10		;multiply r20 value by 10
add		r1,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonPush
add		r1,r20		;add in LSD of count -- data is now complete

ldi		r16,218
out 	OCR1AH,r16
out		OCR1AL,r1	;put new compare register A data in place

pop		r1
ret


;-------------------------------------

SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoGreenFlash
rcall	ButtonPush
sts		$64,r20		;seconds of correction (zero to 255)

rcall	OneGreenFlash
rcall	ButtonPush	;enter interval from 0 to 255 - zero = no correction
sts		$65,r20
sts		$66,r20		;put value both in perm and temp store areas

ret

;---------------------

SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required

push	r1		;minutes accumulator
push	r2		;hours accumulator
push	r3		;day of week accumulator
push	r26
push	r27

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,5		;set up for FIVE green flashes
rcall	FlashGLED	;indicate that five entries are needed

rcall	ButtonPush
mov		r3,r20		;day of week (1=Sunday, 7=Saturday)

rcall	OneGreenFlash
rcall	ButtonPush	;enter MSD of hours 

rcall	Mult10		;multiply value by 10 and store in r2
mov		r2,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonPush	;enter LSD of hours
add		r2,r20		;value in hours now complete in r2

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonPush	;enter msd of minutes
rcall	Mult10		;multiply by 10 and store in r1
mov		r1,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonPush
add		r1,r20		;value in minutes now complete


SetC1:

tst		r2		;total hours count
breq	SetC2		;done
adiw	r26,60
dec		r2
rjmp	SetC1


SetC2:

add		r26,r1		;total minutes count
brcs	SetC3
rjmp	SetEnd


SetC3:

inc		r27		;carry flag was set
rjmp	SetEnd


SetEnd:

bclr	7		;turn off global interrupts

sts		$63,r3		;store day-of-week
sts		$61,r26		;store low byte in $61
sts		$62,r27		;store high byte in $62
clr		r15		;zero seconds counter

bset	7		;turn on global interrupts

pop		r27
pop		r26
pop		r3
pop		r2
pop		r1

ret

;--------------------------------------


FlashGLED:		;Flash Green LED no. of times in R16

push	r0
mov		r0,r16

tst		r16
breq	FlashGLEDend

FlashGLEDa:

sbi		PORTB,1			;Turn on GREEN LED
rcall	Delay165
cbi		PORTB,1			;Off GREEN LED
rcall	Delay495
dec		r0
brne	FlashGLEDa

FlashGLEDend:

pop		r0

ret

;------------------------------------


DoubleGreenFlash:

sbi		PortB,1		;on green LED
rcall	Delay165
cbi		PortB,1		;off green LED
rcall	Delay165


OneGreenFlash:

sbi		PortB,1		;on green LED
rcall	Delay165	
cbi		PortB,1		;off green LED

ret

;------------------------------------


TwoGreenFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashGLED
ret

;-------------------------------------


;Delay1 subroutines


DelayPoint1:		;.1 second delay

push 	r1
push 	r16

clr		r1
ldi		r16,233


DP1a:

nop
nop
nop
dec		r1
brne	DP1a
dec		r16	
brne	DP1a

pop		r16
pop		r1
	
ret

;--------------------


Delay2310:

push	r31
ldi		r31,24
rjmp	DelayStart


Delay40th:		; 1/40th second delay
			; r31 is a reserved variable
push	r31
push	r0
push	r1
ldi		r31,116
mov		r1,r31		; set up r1 for 1/40th second delay
ldi		r31,1
clr		r0

rjmp	DelayA


Delay495:

push	r31
ldi		r31,6
rjmp	DelayStart


Delay330:

push	r31
ldi		r31,2
rjmp	DelayStart


Delay165:

push	r31
ldi		r31,1


DelayStart:

push	r0
push	r1

clr		r0
clr		r1		;zero registers

DelayA:

dec		R0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret

;------------------------


Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr		r1	;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100	;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1	;for return of value
pop		r1
ret


;--------------------------------------

Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr		r1		;zero R1

tst		R20		;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16

Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a

Mult10b:

mov		r20,r1		;move r1 value into r16
pop		r1
ret

;-----------------------------


EnterAlarmTimeData:

rcall	ZeroCommandMemory


EnterAlarmTimeData2:

			;manually enters time commands into command store
			;EIGHT entries required;
			; 1) 0=next entry, 1=toggle command, 2=OFF command
			; 2) 0= every day, 1 - 7 = Sunday through Saturday
			; 3) MSD unit number,   4) LSD unit number
			; 5) MSD hour, 6)LSD hour
			; 7) MSD minutes,   8) LSD minutes
			
			
						
push	r0
push	r1
push	r25		;scratch
push	r26		;push X register
push	r27			
push	r30
push	r31


EATD1:

lds		r26,$67		;get bookmark for command memory pointer
			; put in "X," r26, register

cpi		r26,$AE		;check for end of memory storage
brsh	EATD2		;exit if count is same or higher
rjmp	EATD3		;entry okay, ... continue


EATD2:			;give long RED flash showing memory is full ERROR

sbi		PORTB,0		;on LED
rcall	Delay2310	
cbi		PORTB,0		;off LED

rjmp	EATDEnd


EATD3:

ldi		r16,8		;set up for EIGHT green flashes
rcall	FlashGLED	;indicate that eight entries are needed


rcall	ButtonPush	;Enter 0 to exit, 1 for toggle, 2 for OFF
cpi		r20,0
brne	EATD4
rjmp	EATDEnd		;it is an EXIT command


EATD4:

clr		r25		;clear r25 (unit number data register)
sbrs	r20,1		;skip if bit 1(2) is set (1) (off command)
rjmp	EATD5

inc		r25		;put ONE in r25 for off command (to be added later)


EATD5:

rcall	OneGreenFlash	;indicate single entry is needed

rcall	ButtonPush	;Enter 0=every day, 1 - 7 = Sunday through Saturday
lsl		r20
lsl		r20
lsl		r20
mov		r27,r20		;position and store day-of-week command in r27

rcall	DoubleGreenFlash	;indicate two entries upcoming

rcall	ButtonPush	;Enter MSD of Unit Number
rcall	Mult10		;multiply MSD by ten
mov		r0,r20		;save in r0

rcall	OneGreenFlash	;one more entry needed

rcall	ButtonPush	;Enter LSD of Unit Number
add		r0,r20		;add r20 to r0 - unit number now complete
lsl		r0		;multiply unit value by 2 for correct code
add		r0,r25		;add r25 (one if it is off command, zero otherwise)
			;unit code now complete

rcall	DoubleGreenFlash	;indicate TWO entries needed

clr		r30
clr		r31		;zero register for following process

rcall	ButtonPush	;Enter MSD of hours
rcall	Mult10		;multiply r20 by ten
mov		r1,r20		;store in r1

rcall	OneGreenFlash	;one more entry needed

rcall	ButtonPush	;get LSD of hours
add		r1,r20


EATD5a:

tst		r1		
breq	EATD5b


EATD5aa:

adiw	r30,60		;add 60 minutes for number of times in r1
dec		r1	
brne	EATD5aa


EATD5b:

rcall	DoubleGreenFlash	;two entries for minutes are needed

rcall	ButtonPush	;Enter MSD of minutes
rcall	Mult10		;multiply value by ten
mov		r1,r20		;store in r1 temporarily
	
rcall	OneGreenFlash	;indicate one more entry needed

rcall	ButtonPush	
add		r1,r20		;add LSD value of minutes to r1 - min now complete

add		r30,r1		;add r1 to r30 for complete number of minutes
brcc	EATD5c

inc		r31		;add in carried digit to MSD of minutes count


EATD5c:			;completed data of total minutes ready for store

bclr	7		;turn off interrupts

st		x+,r30		;store LSD of minutes in alarm memory - post incr
or		r31,r27		;or day-of-week information into MSD area
st		x+,r31		;store MSD + day-of-week information - post incr
st		x+,r0		;store unit number and on/off command - post incr

bset	7		;turn interrupts back on

sts		$67,r26		;store x register, r26, -- new bookmark

rjmp	EATD1		;go back for new entry


EATDEnd:

pop		r31
pop		r30
pop		r27
pop		r26
pop		r25
pop		r1
pop		r0

ret

;------------------------------


UsecPulse:		; 50 microsecond pulse for power/data line

ldi		r16,60		;load for 50 usec pulse width
sbi		PortB,3		;pulse on

UsecPulse1:

dec		r16
brne	UsecPulse1

cbi		PortB,3		;pulse off

ret

;---------------------------------------


Wag:

sbi		PortB,1		;on green led (to give one wag)
rcall	Delay165
cbi		PortB,1		;off green led
rcall	Delay165
sbi		PortB,0		;on red led
rcall	Delay165
cbi		PortB,0		;off red led
rcall	Delay495

ret

;------------------------------------------


AtomicClockReset:	;resets clock to signal from Atomic Clock at 4:00 a.m.

sbic	PIND,0x03		;skip if 4:00 a.m. reset signal is present
ret				;signal not present

push	r16

lds		r16,$B2		;atomic clock DELAY countdown
tst		r16
brne	ACREnd

ldi		r16,3		;Set a delay so repeat cycles are not done
sts		$B2,r16		;set three minutes (3) delay, to prevent retrigger.

			;This section resets day-of-week to 2 (Monday)
			
bclr	7		;disable global interrupts

clr		r15		;zero seconds counter

ldi		r16,240
sts		$61,r16	
clr		r16
sts		$62,r16		;put 4 into hours (4 a.m.)

bset	7		;restore global interrupts

lds		r16,$B3		;load Atomic Clock DaysMissedDaysCounter count
cpi		r16,3	
brlo	ACR1		;branch if lower than 3 (3 = Monday)

ldi		r16,2		;put 2 (Monday) into Day of Week Counter, R25
sts		$63,r16		; -- a weekend of two days without alarm has passed


ACR1:

clr		r16		;zero Atomic DaysMissedCounter	
sts		$B3,r16		; and store away


ACREnd:

pop		r16
ret

;-----------------------------------------


NightDetect:		;used during interrupt new second section

inc		r22		;increment 4:16 counter
cpi		r22,255
brne	ND1

sbi		PortB,5		;on DC line to CDS photocell circuit


ND1:

cpi		r22,0		
breq	ND2

ret


ND2:

sbrs	r18,0		;skip if bit 0(1) is set (it is daylight)
rjmp	ND3

; --------------   It is Daylight

sbic	PIND,4		;skip if bit 4(16) is clear (it is night)
rjmp	Exit7X		;it is night so Exit7X

dec		r8		;decrement 7X counter, r8
brne	Exit

cbr		r18,1		;clear daylight flag bit 0(1) bit to night
lds		r16,$B4		;get unit number command (X2)
sts		$B5,r16		;put value in $B5 for execution later
clr		r9		;set r9 (flag to execute command)
inc		r9

Exit7X:

ldi		r16,7
mov		r8,r16		;set 7 into r8, 7X counter


Exit:

cbi		PortB,5
ret

;----------------------------


ND3:			;It is NIGHT

sbis	PIND,4		;skit if bit 4(16) is set (it is daytime)
rjmp	Exit7X		;it is night so Exit7X

dec		r8		;decrement r8, 7X counter
brne	Exit

sbr		r18,1		;set r18, bit 0(1), daylight flag to daylight (1)
lds		r16,$B4		;get unit number command from SRAM
inc		r16		;increment to make it an OFF command
sts		$B5,r16		;put value into $B5 for execution later
clr		r9		;set execution flag, r9
inc		r9

rjmp	Exit7X

;--------------------------------------
